/* ************************************************************************************** */
/* Program 		: Liquidity_Pontiff_Singla_2.sas                                          */
/* Date Updated	: Jun 2019                                                                */
/* Author  		: Jeffrey Pontiff & Rohit Singla                                          */
/* Paper   		: Pontiff, Jeffrey and Rohit Singla, Liquidity Risk? (Feb 2019). 	      */
/*				  Critical Finance Review (CFR) journal					                  */
/*			 	  Available at http://dx.doi.org/10.2139/ssrn.3320011					  */
/*			 	  PERMISSION TO USE THIS PROGRAM IS GRANTED UNDER THE CONDITION 		  */
/*			 	  THAT THE USER(S)CITE PONTIFF AND SINGLA'S CFR PUBLICATION	  		      */
/* Summary 		: This program produces the information that is an input to Pontiff Singla*/	
/*				  Table 5, and data that is necessary for Liquidity_Pontiff_Singla_3.sas  */	
/*                This program was designed to be executed on the WRDS mainframe.	      */
/*				  This program requires the final_liquidity data from                     */
/*				  Liquidity_Pontiff_Singla_1.sas, data from WRDS, and data produced with  */
/*                code from Shane Corwin (hlsprd_mo_sample). For the Corwin code, please  */
/*                go here, https://www3.nd.edu/~scorwin/                                  */
/* ************************************************************************************** */

*Setting libraries and options;
%include '!sasroot/wrdslib.sas';
libname home '/scratch/bc/pontiff';
options mprint notes source;

data home.msi  (keep=date year month vwretd);
set crsp.msi ;
month=month(date);
year=year(date);   
run;           

data a;
set crsp.dsf (keep=permno date ret prc vol shrout);
prc=abs(prc);
proc sort data=a;by date;
data b;
set crsp.dsi (keep=date ewretd vwretd);
 abs=abs(vwretd);
 year=year(date);
 month=month(date);
proc means data=b noprint;
by year month;
  var vwretd;
output out=home.mkt_std (keep=year month mkt_std) std=mkt_std;
data home.mkt_std;
set home.mkt_std;
   by year month;
   mkt_std_ch=mkt_std-lag(mkt_std);
data b;
merge a b;
  by date;
if vol ne 0 then do; 
me=abs(ret-ewretd)*shrout/vol;
ya=abs(ret)/vol;
end;
if vol=0 then vol=.;
zero_ret=0;
if ret=0 then zero_ret=1;
drop  shrout ret ewretd;
year=year(date);
month=month(date);
proc sort; by permno year month;

proc means noprint data=b;;
by permno year month;
var prc me ya vol zero_ret;
output out=averages (drop=x1-x3 _type_ _freq_)  mean=mprc newliq amihud mvol mzero n=x1-x3  nvol nret;

proc sort data=averages (keep=year month  nvol nret) out=max;
by year month nvol;

data max;
set max;
  by year month nvol;
  if last.month;
  maxday=nvol;
  keep year month maxday;

proc sort data=home.hlsprd_mo_sample;
   by permno year month;

data averages;
merge averages home.hlsprd_mo_sample;
by permno year month;
proc sort; by year month;
data averages;
merge averages max;
    by year month;
**we are arranging the signs on the variables so inference is consistent;
bas=-1*mspread_0/mprc;
liquid=-1*newliq*maxday/nvol;
trade= (nvol/maxday);
amihud=-1*amihud;
zero=(1-mzero);

data monthly;
set crsp.msf (keep=date permno ret shrout prc vol);
year=year(date);
month=month(date);
mktcap=abs(prc)*shrout;
proc sort; by permno year month;
data monthly;
set monthly;
   by permno year month;
lmktcap=lag(mktcap);
dvol=abs(prc)*vol;
ldvol=lag(dvol);
if ldvol ne 0 then chdvol=(dvol/ldvol)-1;
if permno ne lag(permno) then do; lmktcap=.; ldvol=.; end;
drop date prc shrout dvol ldvol;
proc sort; by permno year month;

data  good;
set crsp.mse (keep=permno date shrcd exchcd);
where shrcd ne . or exchcd ne .;
year=year(date);
month=month(date);
drop date;
proc sort; by permno year month;

data full;
merge good monthly;
  by permno year month;
data full2;
set full;
by permno year month;
   retain xshrcd xexchcd;
if first.permno then xshrcd=shrcd;
if not missing(shrcd) then xshrcd=shrcd;
if first.permno then xexchcd=exchcd;
if not missing(exchcd) then xexchcd=exchcd;
drop shrcd exchcd;
rename xshrcd=shrcd xexchcd=exchcd;  
run;
proc sort data=full2; 
where shrcd in (10,11) and exchcd not in (0,3,33);
by permno year month;
proc sort data=averages; by permno year month;

proc sort data=full2 out=forvol (keep= year month chdvol);
by year month;

proc univariate data=forvol noprint;                 
   by year month;
   var chdvol;
 output out=home.volume_ch median=volume_ch mean=mean_volume_ch;
run;


data home.full2 (drop=shrcd maxday mprc newliq );
merge  full2(in=in)  averages;
   by permno year month;
  if in;

%macro blaster(var);
data worker;
set home.full2;
   by permno year month;
  ch_&var=&var-lag(&var);
  if permno ne lag(permno) or 12*year+month ne 12*lag(year)+lag(month)+1 
     then ch_&var=.;
proc sort; by year month;

proc means noprint;
  where lmktcap ne .;
    by year month;
    var ch_&var;
    weight lmktcap;
    output out=home.index_&var (drop=_type_ _freq_) mean=index_&var;

data ff2;
        set ff.factors_monthly;
        year=year(date);
        month=month(date);
proc print;  
  proc sort; by year month;

data indice;
merge ff2 home.index_&var;
   by year month;
data stocks;
set home.full2 (keep=permno year month ret);
proc sort data=stocks; by year month;
data home.stocks;
merge  stocks indice;
    by year month;
proc sort; by permno year month;

%do index=1929 %to 2018;
proc reg noprint data=home.stocks edf outest=est ;
  where &index -4  <= year <= &index and ret ne . ;
  by permno;
  model ret = index_&var mktrf hml smb;
data est;
set est;
   where _in_+_edf_+1 = 60;
   if index_&var ne 0 and mktrf ne 0 and hml ne 0 and smb ne 0;
   keep permno index_&var ;
 
proc rank data=est  out=est2  groups=10;
   var index_&var;;
   ranks rank_&var ;

data est&index;
set est2;
 year=&index+1;
 do month=1 to 12;
  output est&index;;
 end;
%end;
data home.whole;
set est1929-est2018;
if rank_&var=0 or rank_&var=9;
proc sort; by permno year month;

data home.big;
merge home.whole home.full2;
by permno year month;
proc sort; by year month rank_&var;

proc means noprint data=home.big;;
 where lmktcap ne .;
 weight lmktcap;
 by year month rank_&var;
var ret;
 output out=out mean=portret n=n;
data long;
set out;
  if rank_&var=0;
  long=portret;
  longn=n;
data short;
set out;
   if rank_&var=9;
   short=portret;
   shortn=n;
data home.traded_&var;
merge long short;
   by year month;
   trade_&var=long-short;
   drop _freq_ _type_ portret n;

data home.final_&var;
merge home.index_&var home.traded_&var;
 by year month;
%mend;

%blaster(trade);run;
%blaster(amihud);run;
%blaster(bas);run;
%blaster(liquid);run;
%blaster(zero);run;

