/* ************************************************************************************** */
/* Program 		: Liquidity_Pontiff_Singla_3.sas                                           */
/* Date Updated	: Jun 2019                                                                */
/* Author  		: Jeffrey Pontiff & Rohit Singla                                          */
/* Paper   		: Pontiff, Jeffrey and Rohit Singla, Liquidity Risk? (Feb 2019). 	      */
/*				  Accepted in Critical Finance Review (CFR) journal					      */
/*			 	  Available at http://dx.doi.org/10.2139/ssrn.3320011					  */
/*			 	  PERMISSION TO USE THIS PROGRAM IS GRANTED UNDER THE CONDITION 		  */
/*			 	  THAT THE USER(S)CITE PONTIFF AND SINGLA'S CFR PUBLICATION	  		      */
/* Summary 		: This program produces the correlations of market returns (for high and   */
/*				  low gamma months) with other variables in Pontiff-Singla Table 3.       */	
/*				  This program was designed for PC SAS. This program requires the         */
/*				  final_liquidity data from Liquidity_Pontiff_Singla_1.sas, market level  */
/*				  data from Liquidity_Pontiff_Singla_2.sas, and bond data and the monthly */                
/*                index data from CRSP.                                                   */
/* ************************************************************************************** */

libname data 'C:\Users\pontiff\Documents\Pastor Stambaugh\liquid corr';
libname all  'C:\Users\pontiff\Documents\Pastor Stambaugh\Basic all data';
run;                                                                                                                                    
  data forplot;
  set  all.final_liquidity;                                                                                                      

data bond;
set data.bond;
   date=caldt;
   year=year(caldt);
   month=month(caldt);
data msi;
set data.msi;
  year=year(date);
  month=month(date);
  data for_corr;
merge msi bond data.volume_ch data.mkt_std forplot ;;
  by year month;
  run;
  proc univariate data=for_corr;
   Title 'All';
    var agg_liq;
	run;
	proc univariate data=for_corr;
   Title 'P & S';
    Where 1963<=year<=1999 or (year=1962 and month>=8);
    var agg_liq;
	run;
  proc sort data=for_corr; by descending year descending month;
 data for_corr;
 set for_corr;
by descending year descending month;
 ch_rf=-lag(T30RET) + T30RET;
  run;
  
  data for_corr;
  set for_corr;
  low_l=0;
  **the below agg liq mean and std come from the series that comes from estimation that starts with PS;
  if agg_liq<-0.0280-2*0.0606 then low_l=1;
  proc sort; by low_l;

proc corr data=for_corr;
  var vwretd;
  Title 'P & S';
   Where 1963<=year<=1999 or (year=1962 and month>=8);
  with ch_rf B20RET volume_ch mkt_std mean_volume_ch resid;
proc corr data=for_corr;
  var vwretd;
  title 'not P & S';
  where year<1962 or year>1999 or (year=1962 and month<8);
  with ch_rf B20RET volume_ch mkt_std mean_volume_ch resid;

**now for liquidity sorts;
  
proc corr data=for_corr;
  var vwretd;
  Title 'P & S';
  by low_l;
  Where 1963<=year<=1999 or (year=1962 and month>=8);
  with ch_rf B20RET volume_ch  mean_volume_ch;;
run;

proc corr data=for_corr;
  var vwretd;
   Title 'not P & S';
  where year<1962 or year>1999 or (year=1962 and month<8);
  by low_l;
  with ch_rf B20RET volume_ch  mean_volume_ch;;
  run;
  *now for standard deviation results;
proc corr data=for_corr;
  var agg_liq;
  Title 'P & S';
   Where 1963<=year<=1999 or (year=1962 and month>=8);
  with mkt_std ;
proc corr data=for_corr;
  var agg_liq;
  title 'not P & S';
  where year<1962 or year>1999 or (year=1962 and month<8);
  with  mkt_std ;
  run;
